Biodb dataclass object ft

Biodb dataclass_create ft {
	$object createclass object ft {
		"event" integer references "event"("id") on update cascade on delete cascade
		"source" varchar(20)
		"ftkey" varchar(15)
		"type" varchar(25)
		"name" varchar(50)
		"seq" integer references "seq"("id") on update cascade on delete cascade
		"start" integer
		"end" integer
		"complement" char(1)
		"composite" char(1)
		"partial" char(1)
		"location" varchar(200)
		"confidential" char(1)
		"score" float
		"note" varchar(1000)
		"transl_table" integer
		"proteinseq" varchar(32000)
		"product" varchar(200)
		"gene" varchar(100)
		"protein_id" varchar(100)
		"new" char(1)
		"db_xref" varchar(1000)
		"attr" varchar(32000)
	}
	$object index ft ftkey source type name complement composite product
	$object createtable ft_ref {
		"ft" integer references "ft"("id") on update cascade on delete cascade
		"ref" integer references "ref"("id") on update cascade on delete cascade
	}
	$object createtable ft_attr {
		"ft" integer references "ft"("id") on update cascade on delete cascade
		"attr" varchar(20)
		"value" varchar(1000)
	}
	$object index ft_attr attr
	$object createtable ft_xref {
		"ft" integer references "ft"("id") on update cascade on delete cascade
		"database" varchar(20)
		"id" varchar(20)
	}
	$object createtable ft_parts {
		"ft" integer references "ft"("id") on update cascade on delete cascade
		"pos" integer
		"sub_ft" varchar(50)
	}
	$object index ft_parts pos
}

# composite
# incomplete
# circular

proc bio::biodb_dnaseq {biodb ftid {arg {}}} {
	foreach {table numid} $ftid break
	set db [$biodb db]
	set data [$db exec -flat [subst {select "seq","start","end","complement","composite" from "$table" where "id" = ?}] $numid]
	if {![llength $data]} {
		error "sequence with id \"$numid\" cannot be found in table \"$table\""
	}
	foreach {seq ftstart ftend complement composite} $data break
	if {[string_equal $complement ""]} {set complement 0}
	if {[string_equal $composite ""]} {set composite 0}
	if {![llength $arg]} {
		if {$composite} {
			set parts [$biodb ftparts $ftid]
			set dnaseq ""
			foreach tid $parts {append dnaseq [bio::biodb_dnaseq $biodb $tid]}
		} else {
			set dnaseq [$biodb sequence get $seq $ftstart $ftend]
			if {$complement} {
				set dnaseq [seq_complement -dna $dnaseq]
			}
		}
	} else {
		foreach {first last} $arg {}
		if ![string length $last] {
			set last $first
		}
		if {!$composite} {
			set length [expr {$ftend - $ftstart}]
			if {!$complement} {
				if {[string first end $first] != -1} {
					set first "[string_change $first [list end $length]]"
					set first [expr $first]
				}
				if {[string first end $last] != -1} {
					set last "[string_change $last [list end $length]]"
					set last [expr $last]
				}
				set start [expr {$ftstart + ($first)}]
				set end [expr {$ftstart + ($last)}]
				set dnaseq [$biodb sequence get $seq $start $end]
			} else {
				if {[string first end $first] != -1} {
					set first "[string_change $first [list end $length]]"
					set first [expr $first]
				}
				if {[string first end $last] != -1} {
					set last "[string_change $last [list end $length]]"
					set last [expr $last]
				}
				set end [expr {$ftend - $first}]
				set start [expr {$ftend - $last}]
				set dnaseq [$biodb sequence get $seq $start $end]
				set dnaseq [seq_complement -dna $dnaseq]
			}
		} else {
			set parts [$biodb ftparts $ftid]
			set dnaseq ""
			foreach tid $parts {append dnaseq [bio::biodb_dnaseq $biodb $tid]}
			set length [string length $dnaseq]
			if {[string first end $first] != -1} {
				set first "[string_change $first [list end $length-1]]"
				set first [expr $first]
			}
			if {[string first end $last] != -1} {
				set last "[string_change $last [list end $length-1]]"
				set last [expr $last]
			}
			set dnaseq [string range $dnaseq $first $last]
			#first part
			if {$first < 0} {
				set pre [::bio::biodb_dnaseq $biodb [lindex $parts 0] [list $first -1]]
				set dnaseq $pre$dnaseq
			}
			# last part
			if {$last > $length} {
				append dnaseq [::bio::biodb_dnaseq $biodb [lindex $parts end] [list end+1 end+[expr {$last-$length+1}]]]
			}
		}
	}
	return $dnaseq
}

Biodb datamethod ft sequence {option id args} {
	switch -- $option {
		get {
			set seq [::bio::biodb_dnaseq $object $id $args]
			return $seq
		}
		getrna {
			set seq [::bio::biodb_dnaseq $object $id $args]
			return [string_change $seq {t u T U}]
		}
		getprotein {
			set transtable [db get $id transl_table]
			set seq [::bio::biodb_dnaseq $object $id $args]
			return [seq_translate $seq $transtable]
		}
		default {
			error "unknown option \"$option\""
		}
	}
}

Biodb datamethod ft ftparts {id args} {
	private $object db
	set len [llength $args]
	if {$len == 0} {
		return [$db exec -flat {select "sub_ft" from "ft_parts" where "ft"= ? order by "pos"} $numid]
	} elseif {$len == 1} {
		set ftparts [lindex $args 0]
		$db begin
		if {[catch {
			$db exec {delete from "ft_parts" where "ft"= ?} $numid
			if {[llength $ftparts]} {
				set num 1
				set end -1
				set start -1
				set complement 0
				foreach subft $ftparts {
					if {[catch {$db get $subft start end complement partial} list]} {
						error "subfeature \"$subft\" does not exist"
					}
					foreach {estart eend complement epartial} $list break
					if {($start == -1) || ($estart < $start)} {set start $estart}
					if {$eend > $end} {set end $eend}
					if {![string equal $epartial {}] && ![string equal $epartial { }]} {
						set partial $epartial
					}
					$db exec {insert into "ft_parts" ("ft","pos","sub_ft") values(?,?,?)} $numid $num $subft
					incr num
				}
				$db set -nullvalue {} $id composite 1 start $start end $end complement $complement partial [get partial ""]
			} else {
				foreach {start end} [$db get $id start end] break
				$db set $id composite 0
				catch {$db set $id length [expr {$end-$start+1}]}
			}
		} result]} {
			$db rollback
			return -code error -errorinfo $::errorInfo $result
		} else {
			$db commit
		}
	
	} else {
		error "wrong # args: should be \"$object ftparts ft ?ftparts?\""
	}
}
