package require Class
namespace eval biodb {}

Class subclass Biodb

Biodb method init {} {
	private $object db setfilter getfilter
	private $object segmentsize
	set setfilter [afilter new]
	set getfilter [afilter new]
	set segmentsize 32000
	upvar #0 ::biodb::num num
	if {![info exists num]} {set num 1}
	set db ::biodb::db$num
	incr num
	return $db
}

Biodb method destroy {} {
	private $object db
	::bio::setfiler::$object destroy
	::bio::getfiler::$object destroy
	$db destroy
}

Biodb method begin {} {
	private $object db
	$db begin
}

Biodb method commit {} {
	private $object db
	$db commit
}

Biodb method rollback {} {
	private $object db
	$db rollback
}

Biodb method drop {database args} {
	private $object db meta 
	file delete -force $database
}

Biodb method create_backend {database metaVar} {
	error "create_backend should be implemented by subclass"
}

Biodb method create {database args} {
	private $object db meta
	if {[file exists $database]} {
		error "database \"$database\" already exists"
	}
	set meta(-type) sqlite
	array set meta $args
	auto_load Biodb-$meta(-type)
	if [catch {Biodb-$meta(-type) info class} result] {
		error "-type $meta(-type) not present"
	}
	set meta(version) 0.10
	catch {$db close}
	$object changeclass Biodb-$meta(-type)
	$object create_backend $database meta
	$object changeclass Biodb
	unset meta
}

Biodb method version {} {
	catch {$db close}
	private $object db meta
	return $meta(version)
}

Biodb method open_backend {database args} {
	error "open_backend should be implemented by subclass"
}

Biodb method open {database args} {
	private $object db meta datadir
	if {[info exists meta]} {
		error "other database already open, close first"
	}
	set meta(-type) sqlite	
	array set meta $args
	$object changeclass Biodb-$meta(-type)
	eval {$object open_backend $database meta}
	set datadir $database
	return [$object version]
}

Biodb method close {} {
	private $object db meta
	catch {unset meta}
	catch {$db close}
	$db destroy
	$object changeclass Biodb
}

Biodb method tables {} {
	private $object db
	$db tables
}

Biodb method fields {table} {
	private $object db
	$db fields $table
}

Biodb method insert {table args} {
	private $object db
	if {[string equal [lindex $args 0] -nullvalue]} {
		set nullvalue [lindex $args 1]
		set args [lrange $args 2 end]
	}
	if {![llength $args]} {return}
	set keys [list_unmerge $args 1 values]
	set param [join [list_fill [llength $values] ?] ,]
	set sql [subst {insert into "$table" ("[join $keys \",\"]") values ($param)}]
	if {[info exists nullvalue]} {
		eval {$db exec -nullvalue $nullvalue $sql} $values
	} else {
		eval {$db exec $sql} $values
	}
}

Biodb method set {id args} {
	private $object cache db
	foreach {table id} $id break
	if {![info exists cache(sc,$table)]} {
		set sc [lindex [$db exec -flat {select "superclass" from "class" where "id" = ?} $table] 0]
		if {![string equal $sc ""]} {
			set cache(sc,$table) $sc
		} else {
			set cache(sc,$table) $table
			set sc $table
		}
	} else {
		set sc $cache(sc,$table)
	}
	if {[string equal [lindex $args 0] -nullvalue]} {
		set nullvalue [lindex $args 1]
		set args [lrange $args 2 end]
	}
	if {![llength $args]} {return}
	set keys [list_unmerge $args 1 values]
	if {[info exists cache(id,$sc,$id)]} {
		set param [join [list_fill [llength $values] ?] ,]
		set sql [subst {insert into "$table" ("id","[join $keys \",\"]") values (?,$param)}]
		if {[info exists nullvalue]} {
			eval {$db exec -nullvalue $nullvalue $sql $id} $values
		} else {
			eval {$db exec $sql $id} $values
		}
		unset cache(id,$sc,$id)
	} else {
		set sql [subst {
			update "$table" set
			"[join $keys {" = ?, "}]" = ?
			where "id" = ?
		}]
		if {[info exists nullvalue]} {
			eval {$db exec -nullvalue $nullvalue $sql} $values {$id}
		} else {
			eval {$db exec $sql} $values {$id}
		}
	}
}

#Biodb method delete {id} {
#	private $object db
#	foreach {table id} $id break
#	set sql "delete from \"$table\" where \"id\" = ?"
#	$db exec $sql $id
#}

Biodb method unset {id args} {
	private $object db
	foreach {table id} $id break
	set sql "update \"$table\"\n"
	set pre set
	set values {}
	foreach key $args {
		append sql "$pre \"$key\" = NULL"
		set pre ,
	}
	append sql " where \"id\" = ?"
	eval {$db exec $sql} {$id}
}

Biodb method get {id args} {
	private $object db
	foreach {table id} $id break
	set len [llength $args]
	if {!$len} {
		set values [$db exec -flat "select * from \"$table\" where \"id\" = ?" $id]
		if {![llength $values]} {
			error "object \"$table $id\" not found"
		}
		set fields [$db fields $table]
		return [list_merge $fields $values]
	} else {
		set field [lindex $args 0]
		$db exec -usefetch "select \"$field\" from \"$table\" where \"id\" = ?" $id
		if {($len == 2)&&([$db fetch isnull 0 0])} {
			return [lindex $args 1]
		}
		set value [$db fetch 0]
		if {![llength $value]} {
			error "object \"$table $id\" not found"
		}
		return $value
	}
}

Biodb method select {table {query {}} {fields *} args} {
	private $object db
	if {![string equal $fields *]} {
		set fields \"[join $fields \",\"]\"
	}
	if {![string equal $query {}]} {
		set query "where $query"
	}
	set sql [subst {select $fields from "$table" $query}]
	eval {$db exec} $args {$sql}
}

Biodb method find {tables {query {}} args} {
putsvars tables query args
	private $object db
	set result {}
	if {![string equal $query {}]} {
		set query "where $query"
	}
	foreach table $tables {
		if {[llength $args]} {
			set ids [eval {$db exec [subst {select "id" from "$table" $query}]} $args]
		} else {
			set ids [$db exec [subst {select "id" from "$table" $query}]]
		}
		foreach id $ids {
			lappend result [list $table $id]
		}
	}
	return $result
}

Biodb method orgid {name} {
	private $object db
	set id [$db exec -flat {select "id" from "organism" where "name" = ?} $name]
	if {![isint $id]} {error "orgnanism \"$name\" not found"}
	return [list organism $id]
}

Biodb method event {args} {
	private $object db
	set id [$object new event]
	eval {$object set $id time [clock format [clock seconds] -format "%Y-%m-%d %H:%M:%S"]} $args
	return $id
}

Biodb method new {table} {
	private $object cache db meta
	if {![info exists cache(sc,$table)]} {
		set sc [lindex [$db exec -flat {select "superclass" from "class" where "id" = ?} $table] 0]
		if {![string equal $sc ""]} {
			set cache(sc,$table) $sc
		} else {
			set cache(sc,$table) $table
			set sc $table
		}
	} else {
		set sc $cache(sc,$table)
	}
	set id [$db serial next $sc id]
	set cache(id,$sc,$id) n
	return [list $table $id]
}

Biodb method createtable {table def} {
	private $object db
	regsub -all \n [string trimright [string trimleft $def "\n\t "] "\n\t "] ,\n def
	set sql [subst {create table "$table" ($def)}]
	$db exec $sql
}

#Biodb method createclass {table idtype def {superclass NULL}} {
#	private $object db meta
#	regsub -all \n [string trimright [string trimleft $def "\n\t "] "\n\t "] ,\n udef
#	set sql [subst {create table "$table" (\n\t"id" $idtype not null primary key,\n\t$udef)}]
#	$db begin
#	if {[catch {
#		$db exec $sql
#	} error]} {
#		$db rollback
#		error $error
#	}
#	if {[string equal $idtype integer]} {
#		$db serial add $table id
#	}
#	$db commit
#}
#
#Biodb method createclass {superclass tclass def} {
#	private $object db meta
#	# create subclass data table
#	array set a [$db info table $superclass]
#	if {![info exists a(primary,id)]} {error "superclass \"$superclass\" must have a primary key on the field \"id\""}
#	set type $a(type,id)
#	if {[string equal $type varchar]} {append type ($a(length,id))}
#	set table _${superclass}_$tclass
#	$object createtable $table [subst {
#		"id" $type not null unique references "${superclass}"("id") on update cascade on delete cascade
#		[string trimleft $def "\t\n "]}]
#	# put class in database
#	$db exec {
#		insert into "class" ("id","superclass")
#		values (?,?)
#	} $tclass $superclass
#	set temp [$db exec {select "num","subclass" from "o$subclass" where "id" = ? order by "num"} $tclass]
#	set num [lindex [lindex $temp end] 0]
#	set args [list_subindex $temp 1]
#	lappend args $table
#	if {![isint $num]} {set num 1}
#	$db exec {
#		insert into "o$subclass" ("id","num","subclass")
#		values (?,?,?)
#	} $tclass $num $table
#	set superclassfields [$db fields $superclass]
#	# create view
#	set fields [list_remove $superclassfields id]
#	foreach table $args {
#		eval lappend fields [list_remove [$db fields $table] id]
#	}
#	set sql "create view \"$tclass\" as\nselect \"$superclass\".\"id\" as \"id\""
#	foreach field $fields {
#		append sql ",\n\t\"$field\" as \"$field\""
#	}
#	append sql	"\nfrom \"$superclass\""
#	foreach table $args {
#		append sql [subst { join "$table" on "$table"."id" = "$superclass"."id"}]
#	}
#	$db exec $sql
#	# insert trigger
#	set sql [subst {
#		create trigger "I_${tclass}" instead of insert on $tclass
#		begin
#			insert into "$superclass" ("[join $superclassfields \",\"]","class")
#			values (NEW."[join $superclassfields \",NEW.\"]",'$tclass');
#	}]
#	foreach table $args {
#		set fields [$db fields $table]
#		append sql [subst {
#				insert into "$table" ("[join $fields \",\"]") values (NEW."[join [lreplace $fields 0 0 id] \",NEW.\"]");
#		}]
#	}
#	append sql end
#	$db exec $sql
#	# update trigger
#	set name U_${tclass}
#	set sql [subst {create trigger "$name" instead of update on $tclass}]
#	set comma 0
#	append sql "\nbegin\nupdate \"$superclass\""
#	foreach field $superclassfields {
#		if ($comma) {
#			append sql ",\n\t"
#		} else {
#			append sql "\n\tset "
#			set comma 1
#		}
#		append sql "\"$field\" = NEW.\"$field\""
#	}
#	append sql "\n\twhere \"id\" = OLD.\"id\";\n"
#	foreach table $args {
#		set comma 0
#		append sql "update \"$table\""
#		set fields [$db fields $table]
#		foreach field $fields {
#			if ($comma) {
#				append sql ",\n\t"
#			} else {
#				append sql "\n\tset "
#				set comma 1
#			}
#			append sql "\"$field\" = NEW.\"$field\""
#		}
#		append sql "\n\twhere \"id\" = OLD.\"id\";\n"
#	}
#	append sql end
#	catch {db exec "drop trigger \"$name\""}
#	$db exec $sql
#	# delete trigger
#	set name D_${tclass}
#	set sql [subst {create trigger "$name" instead of delete on $tclass}]
#	set comma 0
#	append sql "\nbegin\n"
#	foreach table $args {
#		set comma 0
#		append sql "delete from \"$table\""
#		append sql "\n\twhere \"id\" = OLD.\"id\";\n"
#	}
#	append sql "delete from \"$superclass\""
#	append sql "\n\twhere \"id\" = OLD.\"id\";\n"
#	append sql end
#	catch {$db exec "drop trigger \"$name\""}
#	$db exec $sql
#}
#
Biodb method subclasses {superclass} {
	private $object db
	set subclasses [$db exec -flat {select "id" from "class" where "superclass" = ?} $superclass]
	foreach subclass $subclasses {
		eval lappend subclasses [$object subclasses $subclass]
	}
	return $subclasses
}

Biodb method class {id} {
	db get [list object* [lindex $id end]] class
}

#Biodb method psequence {option id args} {
#	private $object db datadir
#	foreach {table id} $id break
#	if {[string equal $table seq]} {
#		set data [$db exec -flat [subst {select "id","length" from "$table" where "id" = ?}] $id]
#		if {[string equal $data {}]} {
#			error "sequence with id \"$id\" cannot be found in table \"seq\""
#		}
#		set len [llength $args]
#		switch -- $option {
#			get {
#				set start 0
#				set end [lindex $data 1]
#				if {$len > 2} {
#					error "wrong # args: should be \"$object sequence get {table id} ?start? ?end?\""
#				}
#				foreach {start end} $args break
#				set pstart [expr {$start % 5000}]
#				set pend [expr {$pstart + $end - $start}]
#				set seq [$db exec -flat {
#						select "sequence" from "seq_part" 
#						where "seq" = ? and "pos" >= ? and "pos" <= ? order by "pos"
#				} $id [expr {$start-4999}] [expr {$end+20000}]]
#				set seq [join $seq ""]
#				return [string range $seq $pstart $pend]
#			}
#			set {
#				if {$len != 1} {
#					error "wrong # args: should be \"$object sequence set {table id} sequencedata\""
#				}
#				set seq [lindex $args 0]
#				set len [string length $seq]
#				$db exec {update "seq" set "length" = ? where "id" = ?} $len $id
#				$db exec {delete from "seq_part" where "seq" = ?} $id
#				for {set pos 0} {$pos < $len} {incr pos 5000} {
#					$db exec -cache {insert into "seq_part" values (?,?,?)} $id $pos [string range $seq $pos [expr {$pos+4999}]]
#				}
#			}
#			pset {
#				if {$len != 1} {
#					error "wrong # args: should be \"$object sequence set {table id} sequencedata\""
#				}
#				set seq [lindex $args 0]
#				set len [string length $seq]
#				$db exec {update "seq" set "length" = ? where "id" = ?} $len $id
#				$db exec {delete from "seq_part" where "seq" = ?} $id
#				for {set pos 0} {$pos < $len} {incr pos 5000} {
#					$db exec {insert into "seq_part" values (?,?,?)} $id $pos [string range $seq $pos [expr {$pos+4999}]]
#				}
#			}
#			append {
#				if {$len != 1} {
#					error "wrong # args: should be \"$object sequence append {table id} sequencedata\""
#				}
#				set length [lindex $data 1]
#				set seq [lindex $args 0]
#				set seqlen [string length $seq]
#				set pstart [expr {$length % 5000}]
#				set pos [expr {$length - $pstart}]
#				set pseq [$db exec {
#					select "sequence" from "seq_part" where "seq" = ? and "pos" = ?
#				} $id $pos]
#				set seqpos [expr {5000-$pstart-1}]
#				append pseq [string range $seq 0 $seqpos]
#				$db exec {
#					update "seq_part" set "sequence" = ? where "seq" = ? and "pos" = ?
#				} $pseq $id $pos
#				incr seqpos
#				incr pos 5000
#				for {} {$seqpos < $seqlen} {incr seqpos 5000} {
#					$db exec {
#						insert into "seq_part" values (?,?,?)
#					} $id $pos [string range $seq $seqpos [expr {$seqpos+4999}]]
#				}
#				$db exec {update "seq" set "length" = ? where "id" = ?} [expr {$length + [string length $seq]}] $id
#			}
#			dump {
#				if {$len != 1} {
#					error "wrong # args: should be \"$object sequence dump {table id} channel\""
#				}
#				set f [open $file]
#				fcopy $f [lindex $args 0]
#			}
#			channel {
#				set len [llength $args]
#				if {$len != 1} {
#					error "wrong # args: should be \"$object sequence channel {table id} options\""
#				}
#				set f [open $file [lindex $args 0]]
#				return $f
#			}
#			default {
#				error "unknown option \"$option\""
#			}
#		}
#	} else {
#		switch -- $option {
#			get {
#				set seq [::bio::biodb_dnaseq $object $datadir $seqid $args]
#				return $seq
#			}
#			getrna {
#				set seq [::bio::biodb_dnaseq $object $datadir $seqid $args]
#				return [string_change $seq {t u T U}]
#			}
#			getprotein {
#				set transtable [db get $id transl_table 1]
#				set seq [::bio::biodb_dnaseq $object $datadir $seqid $args]
#				return [seq_translate $seq $transtable]
#			}
#			default {
#				error "unknown option \"$option\""
#			}
#		}
#	}
#}

Biodb method db {} {
	private $object db
	return $db
}

Biodb method index {table args} {
	private $object db indexdone meta
	set superclass [lindex [$db exec -flat {select "superclass" from "class" where "id" = ?} $table] 0]
	if {[string length $superclass]} {
		foreach field [$db fields $superclass] {
			set src($field) $superclass
		}
		foreach table [$db exec {select "subclass" from "o$subclass" where "id" = ? order by "num"} $table] {
			foreach field [$db fields $table] {
				set src($field) $table
			}
		}
		foreach field [list_remove $args id] {
			set name $src($field)-$field
			if {[info exists indexdone($name)]} continue
			$db exec [subst {create index "$name" on "$src($field)" ("$field")}]
			set indexdone($name) 1
		}
	} else {
		foreach field [list_remove $args id] {
			$db exec [subst {create index "$table-$field" on "$table" ("$field")}]
			set indexdone($table-$field) 1
		}
	}
}

Biodb method scanid {string {accvar {}}} {
	private $object db
	if {![string equal $accvar {}]} {upvar $accvar acc}
	set acc {}
	# db class $id
	if {[regexp {^([0-9]+)_([^ ]+)} $string temp id acc]} {return [list seq $id]}
	set id [$db exec {select "id" from "seq" where "acc" = ?} [lindex $string 0]]
	if {![string equal $id ""]} {return [list seq $id]}
	set string [file tail $string]
	if {[regexp {^[^.]+} $string temp]} {
		set id [$db exec {select "id" from "seq" where "name" = ?} $temp]
	}
	if {![string equal $id ""]} {return [list seq $id]}
	set id [$db exec {select "id" from "seq" where "name" = ?} $string]
	if {![string equal $id ""]} {return [list seq $id]}
	error "could not find source seq for \"$string\""
}

# support for data objects

Biodb classmethod do_index {} {
	set dir [file join $::bio::dir lib biodb]
	set files [dirglob $dir *.tcl]
	catch {unset methods}
	catch {unset subclasses}
	catch {unset superclass}
	catch {unset classfiles}
	set o [open [file join $dir tclIndex] w]
	puts $o {# Tcl autoload index file, version 2.0}
	puts $o {# This file is generated by the "Biodb do_index" command}
	puts $o {}
	foreach file $files {
		set f [open [file join $dir $file]]
		while {![eof $f]} {
			set line [gets $f]
			if {[regexp {Biodb datamethod ([^ ]+) ([^ ]+)} $line temp class method]} {
				set methods($class,$method) $file
			} elseif {[regexp {Biodb dataclass ([^ ]+) ([^ ]+)} $line temp sclass class]} {
				lappend subclasses($sclass) $class
				set superclass($class) $sclass
				set classfiles($class) $file
			} elseif {[regexp {Biodb dataclass_create ([^ ]+)} $line temp class]} {
				puts $o "set auto_index(::biodb::class_${class}_create) \[list source \[file join \$dir $file\]\]"
			}
		}
	}
	catch {unset allmethods}
	foreach name [array names methods] {
		foreach {class method} [split $name ,] break
		set file $methods($name)
		set allmethods($method) $file
	}
	# make class_create_all
	set code "proc ::biodb::class_create_all object \{\n"
	array set todo [array get superclass]
	foreach c [array names todo] {
		if {![info exists todo($c)]} continue
		set temp {}
		while 1 {
			if {![info exists todo($c)]} break
			lappend temp "\tbiodb::class_${c}_create \$object"
			set newc $todo($c)
			unset todo($c)
			if {$newc eq "object"} break
			set c $newc
		}
		append code [join [list_reverse $temp] \n]\n
	}
	append code "\}"
	file_write [file join $dir class_create_all.tcl] $code
	puts $o "set auto_index(::biodb::class_create_all) \[list source \[file join \$dir class_create_all.tcl\]\]"
	unset -nocomplain todo
	# do methods
	set finallist {}
	set todo [array names subclasses]
	while {[llength $todo]} {
		set list [list_lremove $todo [array names superclass]]
		set todo [list_lremove $todo $list]
		eval lappend finallist $list
		foreach top $list {
			foreach temp $subclasses($top) {unset superclass($temp)}
		}
	}
	foreach sclass $finallist {
		set ms [array names methods $sclass,*]
		set ms [list_regsub -all ^$sclass, $ms {}]
		foreach subclass $subclasses($sclass) {
			foreach method $ms {
				set methods($subclass,$method) $classfiles($subclass)
			}
		}
	}
	foreach name [array names methods] {
		foreach {class method} [split $name ,] break
		set file $methods($name)
		puts $o "set auto_index(::biodb::method_${class}_$method) \[list source \[file join \$dir $file\]\]"
	}
	foreach name [array names classfiles] {
		set file $classfiles($name)
		puts $o "set auto_index(::biodb::class_${name}) \[list source \[file join \$dir $file\]\]"
	}
	foreach method [array names allmethods] {
		set file $allmethods($method)
		puts $o "set auto_index(::Class::Biodb,,m,$method) \[list source \[file join \$dir $file\]\]"
	}
	close $o
}

Biodb classmethod do_load {dataclass} {
	private $class do_data
	if {[string equal $dataclass object]} {
		set do_data(sup,object) {}
		return
	}
	if {![info exists do_data(sup,$dataclass)]} {
		eval $auto_index(::biodb::class_$dataclass)
	}
}

Biodb method actionconfig {dataclass type field args} {
	private $object setaction
	if {[llength $args]} {
		set setaction($dataclass,$type,$field) [lindex $args 0]
	} else {
		return $setaction($dataclass,$type,$field)
	}
}

Biodb classmethod dataclass {superclass dataclass} {
	private $class do_data setaction
	if {![info exists do_data(sup,$superclass)]} {
		$class do_load $superclass
	}
	foreach name [info commands ::biodb::method_${superclass}_*] {
		regsub ^::biodb::method_${superclass}_ $name {} method
		interp alias {} ::biodb::method_${dataclass}_$method {} ::biodb::method_${superclass}_$method
	}
	foreach name [array names setaction $superclass,*] {
		regsub ^${superclass}, $name $dataclas, newname
		set setaction($newname) $setaction($name)
	}
	set do_data(sup,$dataclass) $superclass
	lappend do_data(sub,$superclass) $dataclass
}

Biodb classmethod dataclass_create {dataclass init} {
	proc ::biodb::class_${dataclass}_create {object} $init
}

Biodb classmethod datamethod_rec {dataclass datamethod} {
	private $class do_data
	foreach subclass [get do_data(sub,$dataclass) {}] {
		interp alias {} ::biodb::method_${subclass}_$datamethod {} ::biodb::method_${dataclass}_$datamethod
		Biodb datamethod_rec $subclass $datamethod
	}
}

Biodb classmethod datamethod {dataclass datamethod arg def} {
	set code {
		foreach {dataclass numid} $id break
		if {[catch {::biodb::method_${dataclass}_@datamethod@ $object $dataclass $numid @args@} result]} {
			if {(![llength $id] == 2) || ![isint $numid]} {
				return -code error -errorinfo $::errorInfo "error in format of id \"$id\""
			} elseif {![llength [info commands ::biodb::method_${dataclass}_@datamethod@]]} {
				return -code error -errorinfo $::errorInfo "method \"@datamethod@\" not available for class \"$dataclass\""
			} else {
				return -code error -errorinfo $::errorInfo $result
			}
		}
		return $result
	}
	regsub -all @datamethod@ $code $datamethod code
	regsub -all @args@ $code \$[join $arg " \$"] code
	Biodb method $datamethod $arg $code
	set dataargs {object dataclass numid}
	
	eval lappend dataargs $arg
	set dataargs [list_change $dataargs {args arg}]
	proc ::biodb::method_${dataclass}_$datamethod $dataargs [subst {
	set args \$arg
	$def
	}]
	Biodb datamethod_rec $dataclass $datamethod
}

proc ::biodb::setlink {list data} {
	return [lindex $list end]
}

proc ::biodb::getlink {value table} {
	return [list $table $value]
}

Biodb method link {srctable srcfield desttable} {
	private $object setfilter getfilter
	$setfilter set $srctable,$srcfield ::biodb::setlink $desttable
	$getfilter set $srctable,$srcfield ::biodb::getlink $desttable
}

# extras

Biodb method tocsv {query f} {
	private $object db
	$db exec -usefetch $query
	set fields [$db fetch fields]
	puts $f [join $fields \t]
	while {![catch {$db fetch} line]} {
		puts $f [join $line \t]
	}
}

if 0 {
proc biodb_addtoali {db ali ft {alignto {}}} {
	set aliseq [$db get $ali _aliseq]
	set type [$db get $ali type]
	set alen [$db get $ali length]
	set seq [biodb_${type}seq $db $ft]
	set len [string length $seq]
	set atolen [llength $alignto]
	regsub -all . $seq {\0 } seq
	set seq " [string toupper $seq]"
	if {$atolen == 0} {
		if {$len > $alen} {
			$db set $ali length $alen
			set addlen [expr {$len - $alen}]
			foreach aliseq [$db find [list _ali == $ali]] {
				set seq [$db get $aliseq aliseq]
				append seq [string_fill {- } $addlen]
				$db set $aliseq aliseq $seq
			}
		} else {
			set addlen [expr {$alen - $len}]
			append seq [string_fill {- } $addlen]
		}
	} else {
#		set refseqs [list]
#		if {$atolen == 1} {
#			set refseqs [aliseq_seqonly [$db get $alignto aliseq]]
#		} else {
#			set refseq [list]
#			foreach id $alignto {
#				lappend refseqs [aliseq_nosec [$db get $id aliseq]]
#			}
#		}
#		ali_align -output edit $refseqs $seq
#		ali_align -output edit AGCCGGUCAUAUC ACCGGUCAUAAUC
#		ali_align AGCCGGUCAUAUC ACCGGUCAUAAUC
	}
	set new [$db new $aliseq]
	$db set $new _ali $ali new 1 _ft $ft \
		seqname [$db get $ft seqname] \
		cdate [clock format [clock seconds] -format "%Y-%m-%d %T"] \
		aliseq $seq
	return $new
}
}

