array set conv_gbk_seqdb_featuretrans {
	translation proteinseq
}

proc _conv_gbk_biodb_parsefeatures {biodb f linevar} {
	upvar $linevar line
	set ftlist {}
	set first 1
	while {![eof $f]} {
		set cline $line
		# read a line and the continuation lines
		while {![eof $f]} {
			set line [gets $f]
			if {![string_equal [string index $line 0] { }]} break
			if {![string_equal [string index $line 5] { }]} break
			if {[string_equal [string index $line 21] "/"]} break
			append cline " [string range $line 21 end]"
		}
		# check for the end of the current feature
		if {![string_equal [string index $cline 5] { }]} {
			# handle location of feature
			set key [string trimright [string range $cline 5 19]]
			set location [string range $cline 21 end]
			set loc [_conv_embl_biodb_parselocation $location]
			if {$first} {
				set first 0
			} else {
				lappend ftlist [array get attrs]
			}
			catch {unset attrs}
			lappend ftlist $key $loc
		} else {
			# handle attributes to the current feature
			set pos [string first = $cline]
			if {$pos != -1} {
				set attr [string range $cline 22 [expr {$pos-1}]]
				if ![string length $attr] continue
				set value [string trimleft [string trimright [string range $cline [expr {$pos+1}] end] "\""] "\""]
				switch -- $attr {
					translation {
						regsub -all " " $value "" value
						set attrs(proteinseq) $value
					}
					citation {
						regexp {\[([0-9]+)\]} $value temp value
						if [info exists rn($value)] {
							set attrs(refnum) $value
						} else {
							set attrs($attr) $value
						}
					}
					organism {
						set attrs(organism) $value
					}
					default {
						lappend attrs($attr) $value
					}
				}
			} else {
				set attr [string range $cline 22 end]
				lappend attrs(switches) $attr
			}
		}
		# check for the end of the feature list
		if {![string_equal [string index $line 0] { }]} break
	}
	lappend ftlist [array get attrs]
	return $ftlist
}

proc conv_gbk_parseentry {biodb f datavar} {
	set db [$biodb db]
	upvar $datavar data
	upvar #0 ::_conv_gbk_biodb_trans trans
	set line [gets $f]
	set ref {}
	catch {unset data}
	set data(reflist) {}
	set data(ftlist) {}
	set xref {}
	set updated 0
	while {![eof $f]} {
		set lkey [string trimleft [string trimright [string range $line 0 10]]]
		if {[inlist {ORIGIN CONTIG} $lkey]} {
			break
		}
		set value [string range $line 12 end]
		while {![eof $f]} {
			set line [gets $f]
			if {("[string index $line 0]" != " ")||("[string index $line 2]" != " ")||("[string index $line 5]" != " ")} break
			append value "\n[string range $line 12 end]"
		}
		switch -- $lkey {
			{} {}
			LOCUS {
				if {![regexp {([^ ]+) +([0-9]+) *bp +([^ ]+) +([^ ]+) +([^ ]+)} $value temp \
					data(locus) temp data(molecule) data(division) data(date)]} {
					set data(locus) [lindex $value 0]
				} else {
					set data(type) [string index $data(molecule) 0]
				}
				if {![info exists data(acc)]} {
					set data(acc) $data(locus)
				}
			}
			DEFINITION {
				regsub -all " *\n *" $value { } value
				set data(def) $value
			}
			ACCESSION {
				set value [string_change $value {; {}}]
				set data(acc) [lindex $value 0]
				if {[llength $value] > 1} {
					set data(oldacc) [lrange $value 1 end]
				}
			}
			NID {
				set data(data) $value
			}
			VERSION {
				foreach {version dat(gi)} $value break
				set data(version) [lindex [split $version .] end]
			}
			KEYWORDS {
				regsub -all " *\n *" [string trimright $value .] { } value
				if {[string length $value]} {
					set data(keywords) [string_split $value "; "]
				}
			}
			SOURCE {
				if {[regexp {^([^()]+) +\(([^()]+)\)$} $value temp data(organism) data(common_name)]} {
				} else {
					set data(organism) $value
				}
			}
			ORGANISM {
				regsub -all " *\n *" [string trimright $value .] { } value
				set data(taxonomy) [string_split $value "; "]
			}
			REFERENCE {
				set refnum [scan $value %d 0]
				if [regexp {([0-9?]+) to ([0-9?]+)} $value temp s e] {
					set refpos [list $s $e]
				}
			}
			MEDLINE {
				# lappend xref rx_MEDLINE $value
			}
			PUBMED {
				# lappend xref rx_PUBMED $value
			}
			AUTHORS {
				set value [string_change $value [list "., " ".#" ",\n" ".#" ". and " ".#" "\;" ""]]
				lappend ref authors [split $value "#"]
			}
			TITLE {
				lappend ref title [string_change $value [list "\n" " "]]
			}
			JOURNAL {
				if [regexp "^Submitted \\((\[^()\]+)\\)\[^\n\]*\n(.*)\$" $value temp date place] {
					catch {set date [time_format [time_scan $date] "%Y-%m-%d"]}
					regsub -all " *\n *" $place { } place
					set ref [map_unset $ref title]
					lappend ref class submission source $place date $date
				} elseif [string match $value Unpublished*] {
					lappend ref class unpublished source Unpublished
				} elseif [regexp {\(in\) ([^();]+)[^;]*;.([^:]+):([^;]+);.([^()]+) \(([0-9?]+)\)} $value temp editors title pages place year] {
					set editors [string_change [string trimright $editors] [list "., " ".#" ",\n" ".#" ". and " ".#" "\;" ""]]
					set editors [split $editors "#"]
					set pages [split $pages -]
					lappend ref class book source $title editors $editors \
						pages_start [lindex $pages 0] pages_end [lindex $pages 1] \
						year $year place $place
				} elseif [regexp {^([^0-9]+) ([^,:]*)[,:] *([^(]*) *\(([^)]*)} $value temp jou vol pages year] {
					set pages [split $pages -]
					if {![regexp {^([0-9]+)\(([0-9]+)\)$} $vol temp vol subvol]} {
						lappend ref class journal source $jou volume $vol \
						pages_start [lindex $pages 0] pages_end [lindex $pages 1] year $year
					} else {
						lappend ref class journal source $jou volume $vol subvolume $subvol \
						pages_start [lindex $pages 0] pages_end [lindex $pages 1] year $year
					}
				} elseif [regexp {^Thesis \(([0-9?]+)\), (.*)$} $value temp year place] {
					regsub -all " *\n *" $place { } place
					lappend ref class thesis place $place year $year
				} elseif [regexp {^Patent number ([^.-]+)-([^/]+)/([^,]+), ([^.]+)\..(.*)} $value temp patent type serial date applicants] {
					set date [time_format [time_scan $date] "%Y-%m-%d"]
					set applicants [string_change [string trimright $applicants " ."] [list "., " ".#" ",\n" ".#" ". and " ".#" "\;" ""]]
					set applicants [split $applicants "#"]
					lappend ref class patent patent $patent type $type serial $serial date $date applicants $applicants
				} else {
					lappend ref class unknown source $value
				}
				lappend data(reflist) $refnum [get refpos {}] $ref
				set ref {}
			}
			COMMENT {
				regsub -all " *\n *" $value { } value
				set data(comment) $value
			}
			FEATURES {
				set data(ftlist) [_conv_gbk_biodb_parsefeatures $biodb $f line]
			}
			"" {
				break
			}
			default {
				log "unknown key $lkey ($value)"
				set data([get trans($lkey) $lkey]) $value
			}
		}
	}
	if {![info exists data(acc)]} {
		error eof
	}
	if {[string_equal $lkey ORIGIN]} {
		# Go for the sequence
		set seq ""
		while {![eof $f]} {
			set line [gets $f]
			if {"$line" == "//"} break
			regsub -all {[^A-Za-z]} $line {} line
			append seq $line
		}
		set len [string length $seq]
	} else {
		set line [gets $f]
		set len -1
		set seq $value
	}
	set data(length) $len
	set data(seq) $seq
	return $data(acc)
}

proc conv_gbk_biodb {args} {
	set opt(-db) gbk
	cmd_args conv_gbk_biodb {
		-source {any "source db of the gbk file (default: gbk)"}
		-log {channel "channel on which to write logs"}
		-name {string "sequence name"}
	} {biodb file} $args
	if {[info exists opt(-log)]} {
		setlog [subst {puts $opt(-log) \$log}]
	} else {
		setlog {}
	}
	set eventid [$biodb event type import sourcetype gbkfile source $file]
	set f [open $file]
	set num 1
	set db [$biodb db]
	# Read gbk data into database
	foreach key [$db fields ft] {
		set ft_fields($key) 1
	}
	while {![eof $f]} {
		if {[catch {conv_gbk_parseentry $biodb $f data} result]} {
			if {[string equal $result eof]} break
			error $result
		}
		# insert references
		catch {unset rnum}
		catch {unset rpos}
		foreach {refnum refpos ref} $data(reflist) {
			log "ref $refnum"
			catch {unset rd}
			array set rd $ref
			set refid {}
			switch $rd(class) {
				submission {
					set refid [$db exec {
						select "id" from "ref" 
						where "class" = ? and "authors" = ? and "date" = ? and "source" = ?
					} $rd(class) $rd(authors) $rd(date) $rd(source)]
				}
				unpublished {
					set refid [$db exec {
						select "id" from "ref" 
						where "class" = ? and "authors" = ? and "title" = ?
					} $rd(class) $rd(authors) $rd(title)]
				}
				book {
					set refid [$db exec {
						select "id" from "ref" 
						where "class" = ? and "authors" = ? and "source" = ? and "editors" = ?
						 and "pages_start" = ? and "pages_end" = ? and "year" = ?
					} $rd(class) $rd(authors) $rd(source) $rd(editors) $rd(pages_start) $rd(pages_end) $rd(year)]
				}
				journal {
					set refid [$db exec -nullvalue {} {
						select "id" from "ref" 
						where "class" = ? and "authors" = ? and "source" = ? and "volume" = ?
						and "subvolume" = ? and "pages_start" = ? and "pages_end" = ?
						and "year" = ?
					} $rd(class) $rd(authors) $rd(source) $rd(volume) [get rd(subvolume) {}] $rd(pages_start) $rd(pages_end) $rd(year)]
				}
				thesis {
					set refid [$db exec {
						select "id" from "ref" 
						where "class" = ? and "authors" = ? and "year" = ?
					} $rd(class) $rd(authors) $rd(year)]
				}
				patent {
					set refid [$db exec {
						select "id" from "ref" 
						where "class" = ? and "patent" = ?
					} $rd(class) $rd(patent)]
				}
				unknown {
					set refid [$db exec {
						select "id" from "ref" 
						where "class" = ? and "authors" = ? and "source" = ?
					} $rd(class) $rd(authors) $rd(source)]
				}
			}
			if {[llength $refid]} {
				set rnum($refnum) $refid
				set rpos($refnum) $refpos
			} else {
				set refid [$biodb new ref]
				eval {$biodb set $refid event [lindex $eventid 1]} $ref
				set rnum($refnum) $refid
				set rpos($refnum) $refpos
			}
		}
		# insert taxonomy
		if {[info exists data(organism)]} {
			set orgid [lindex [$biodb find organism [subst {"name" = '$data(organism)'}]] 0]
			if {![llength $orgid]} {
				set orgid [$biodb new organism]
				set taxonomy [get data(taxonomy)]
				set common_name [get data(common_name) ""]
			} else {
				foreach {taxonomy common_name} [$biodb get $orgid taxonomy common_name] break
				if {![string length $taxonomy]} {set taxonomy [get data(taxonomy) ""]}
				if {![string length $common_name]} {set common_name [get data(common_name) ""]}
			}
			log org:$data(organism)
			$biodb set -nullvalue {} $orgid name $data(organism) taxonomy $taxonomy common_name $common_name
		}
		# insert sequence
		log seq
		if {[info exists opt(-name)]} {
			set name $opt(-name)
		} else {
			set name $data(acc)
		}
		set seqid [lindex [$biodb find seq {"acc" = ? and "db" = ?} $data(acc) $opt(-db)] 0]
		if {[llength $seqid]} {
			set seqid [list seq [lindex $seqid 1]]
			$db exec {delete from "seq_attr" where "seq" = ?} $seqid
			$db exec {delete from "seq_ref" where "seq" = ?} $seqid
			$db exec {delete from "seq_keywords" where "seq" = ?} $seqid
			$db exec {delete from "seq_xref" where "seq" = ?} $seqid
			$biodb set -nullvalue {} $seqid db $opt(-db) event [lindex $eventid 1] type [get data(type) {}] \
				acc $data(acc) version [get data(version) {}] oldacc [get data(oldacc) ""] \
				date [get data(data) {}] release [get data(release)] update [get data(update)] \
				update_release [get data(update_release) ""] update_version [get data(update_version) ""] \
				division [get data(division) ""] organism [get orgid ""] \
				organel [get data(organel) ""] def [get data(def) ""] \
				length [get data(length) ""] \
				time [get data(time) ""] nid [get data(nid) ""] \
				comment [get data(comment) ""] name $name
		} else {
			set seqid [$biodb new seq]
			$biodb set -nullvalue {} $seqid db $opt(-db) event [lindex $eventid 1] type [get data(type) ""] \
				acc $data(acc) version [get data(version) {}] oldacc [get data(oldacc) ""] \
				date [get data(data) {}] release [get data(release)] update [get data(update)] \
				update_release [get data(update_release) ""] update_version [get data(update_version) ""] \
				division [get data(division) ""] organism [get data(organism) ""] \
				organel [get data(organel) ""] def [get data(def) ""] \
				length [get data(length) ""] \
				time [get data(time) ""] nid [get data(nid) ""] \
				comment [get data(comment) ""] name $name
		}
		foreach {refnum refpos ref} [get data(reflist)] {
			$db exec -nullvalue {} {
				insert into "seq_ref"("seq","refnum","ref","refstart","refend") 
				values (?,?,?,?,?)} $seqid  $refnum  $rnum($refnum) [lindex $refpos 0] [lindex $refpos 1]
		}
		foreach line [get data(xref) ""] {
			while {[llength $line] < 3} {lappend line {}}
			eval {$db exec -nullvalue {} {insert into "seq_xref" values (?,?,?,?)} $seqid} [lrange $line 0 2]
		}
		foreach keyw [get data(keywords) ""] {
			$db exec {insert into "seq_keywords" values (?,?)} $seqid $keyw
		}
		log length:[string length $data(seq)]
		$biodb sequence set $seqid $data(seq)
		# insert features
		foreach {ftkey locs attrs} $data(ftlist) {
			log "$ftkey $locs"
			switch -- $ftkey {
				source {
					# nog te herwerken -> source moet apart
					set length [map_get [lindex $locs 0] end]
					incr length
					set seqdata [list length $length]
					foreach {attr value} $attrs {
						switch $attr {
							organism - strain - tissue_type {
								lappend seqdata $attr $value
							}
							sequenced_mol {
								lappend seqdata type [string index $value 0]
							}
							db_xref {
#								if {[regexp {taxon:([0-9]+)} $value temp num]} {
#									$biodb set [list organism $data(organism)] taxonomy_xref $num
#								}
							}
						}
					}
					eval {$biodb set $seqid} $seqdata
				}
				default {
					if {[llength $locs] == 1} {
						# single feature
						array set l [lindex $locs 0]
						set ftid [$db exec {
							select "id" from "ft" 
							where "seq" = ? and ("ftkey" = ? or "ftkey" = '?') 
							and "start" = ? and "end" = ? and "complement" = ?
							order by "ftkey" desc
						} $seqid $ftkey $l(start) $l(end) [get l(complement) 0]]
						if {![llength $ftid]} {
							set ftid [$biodb new ft]
							$biodb set $ftid seq [lindex $seqid 1] ftkey $ftkey \
								start $l(start) end $l(end) complement [get l(complement) 0]
						} else {
							set ftid [list ft [lindex $ftid 0]]
						}
						$biodb set -nullvalue {} $ftid partial [get l(partial) ""] ftkey $ftkey
						# delete parts if it has them
						$biodb ftparts $ftid {}
					} else {
						# composite feature
						catch {unset partial}
						set subfts {}
						switch $ftkey {
							CDS {set subftkey exon}
							default {set subftkey part_$ftkey}
						}
						set num 1
						foreach loc $locs {
							catch {unset l}
							array set l $loc
							set ftseqid $seqid
							if {[info exists l(acc)]} {
								if {![string equal $l(acc) $data(acc)]} {
									set ftseqid [$db exec {
										select "id" from "seq" 
										where "acc" = ? and "db" = ?
									} $l(acc) $opt(-db)]
									if {![llength $ftseqid]} {
										set ftseqid [$biodb new seq]
										$biodb set $ftseqid acc $l(acc) \
											def {placeholder for link from feature} db $opt(-db)
									}
								}
							}
							set subftid [$db exec {
								select "id" from "ft" 
								where "seq" = ? and "start" = ? and "end" = ? and "complement" = ?
								and "ftkey" = ?
							} $ftseqid $l(start) $l(end) [get l(complement) 0] $subftkey]
							if {![llength $subftid]} {
								set subftid [$biodb new ft]
								$biodb set $subftid seq [lindex $ftseqid 1] ftkey $subftkey \
									start $l(start) end $l(end) complement [get l(complement) 0] \
									type $ftkey-part-$num
							} else {
								set subftid [list ft $subftid]
							}
							lappend subfts $subftid
							incr num
						}
						set ftid [$biodb new ft]
						$biodb set $ftid seq [lindex $seqid 1] ftkey $ftkey
						$biodb ftparts $ftid $subfts
					}
					# process attributes
					set keepattrs {}
					set temp {}
					foreach {attr value} $attrs {
						switch $attr {
							default {
								if {[info exists ft_fields($attr)]} {
									lappend temp $attr $value
								} else {
									lappend keepattrs $attr $value
									$db exec {insert into "ft_attr" values(?,?,?)} $ftid $attr $value
								}
							}
						}
					}
					eval {$biodb set -nullvalue {} $ftid attr $keepattrs} $temp
				}
			}
		}
		log "$num $data(acc)"
	}
	$db commit
	close $f
}

