Class subclass GenbankFile

GenbankFile method init {{file {}}} {
	private $object data f
	private $class trans
	array set trans {NI nid OG organel SV version}
	if {[string length $file]} {$object parse $file}
}

GenbankFile method close {} {
	private $object f num
	for {set i 1} {$i < $num} {incr i} {
		upvar #0 [privatevar $object data_$i] data
		unset -nocomplain data
	}
	if {[info exists f]} {
		close $f
		unset f
	}
}

GenbankFile method destroy {} {
	private $object f
	if {[info exists f]} {close $f}
}

GenbankFile method _parseentry {num} {
	private $object data f
	private $class trans
	upvar #0 [privatevar $object data_$num] data
	set line [gets $f]
	set ref {}
	catch {unset data}
	set data(reflist) {}
	set data(ftlist) {}
	set xref {}
	set updated 0
	while {![eof $f]} {
		set lkey [string trimleft [string trimright [string range $line 0 10]]]
		if {[inlist {ORIGIN CONTIG} $lkey]} {
			break
		}
		set value [string range $line 12 end]
		while {![eof $f]} {
			set line [gets $f]
			if {("[string index $line 0]" != " ")||("[string index $line 2]" != " ")||("[string index $line 5]" != " ")} break
			append value "\n[string range $line 12 end]"
		}
		switch -- $lkey {
			{} {}
			LOCUS {
				if {![regexp {([^ ]+) +([0-9]+) *bp +([^ ]+) +([^ ]+) +([^ ]+)} $value temp \
					data(locus) temp data(molecule) data(division) data(date)]} {
					set data(locus) [lindex $value 0]
				} else {
					set data(type) [string index $data(molecule) 0]
				}
				if {![info exists data(acc)]} {
					set data(acc) $data(locus)
				}
			}
			DEFINITION {
				regsub -all " *\n *" $value { } value
				set data(def) $value
			}
			ACCESSION {
				set value [string_change $value {; {}}]
				set data(acc) [lindex $value 0]
				if {[llength $value] > 1} {
					set data(oldacc) [lrange $value 1 end]
				}
			}
			NID {
				set data(data) $value
			}
			VERSION {
				foreach {version dat(gi)} $value break
				set data(version) [lindex [split $version .] end]
			}
			KEYWORDS {
				regsub -all " *\n *" [string trimright $value .] { } value
				if {[string length $value]} {
					set data(keywords) [string_split $value "; "]
				}
			}
			SOURCE {
				if {[regexp {^([^()]+) +\(([^()]+)\)$} $value temp data(organism) data(common_name)]} {
				} else {
					set data(organism) $value
				}
			}
			ORGANISM {
				regsub -all " *\n *" [string trimright $value .] { } value
				set data(taxonomy) [string_split $value "; "]
			}
			REFERENCE {
				set refnum [scan $value %d 0]
				if [regexp {([0-9?]+) to ([0-9?]+)} $value temp s e] {
					set refpos [list $s $e]
				}
			}
			MEDLINE {
				# lappend xref rx_MEDLINE $value
			}
			PUBMED {
				# lappend xref rx_PUBMED $value
			}
			AUTHORS {
				set value [string_change $value [list "., " ".#" ",\n" ".#" ". and " ".#" "\;" ""]]
				lappend ref authors [split $value "#"]
			}
			TITLE {
				lappend ref title [string_change $value [list "\n" " "]]
			}
			JOURNAL {
				if [regexp "^Submitted \\((\[^()\]+)\\)\[^\n\]*\n(.*)\$" $value temp date place] {
					catch {set date [time_format [time_scan $date] "%Y-%m-%d"]}
					regsub -all " *\n *" $place { } place
					set ref [map_unset $ref title]
					lappend ref class submission source $place date $date
				} elseif [string match $value Unpublished*] {
					lappend ref class unpublished source Unpublished
				} elseif [regexp {\(in\) ([^();]+)[^;]*;.([^:]+):([^;]+);.([^()]+) \(([0-9?]+)\)} $value temp editors title pages place year] {
					set editors [string_change [string trimright $editors] [list "., " ".#" ",\n" ".#" ". and " ".#" "\;" ""]]
					set editors [split $editors "#"]
					set pages [split $pages -]
					lappend ref class book source $title editors $editors \
						pages_start [lindex $pages 0] pages_end [lindex $pages 1] \
						year $year place $place
				} elseif [regexp {^([^0-9]+) ([^,:]*)[,:] *([^(]*) *\(([^)]*)} $value temp jou vol pages year] {
					set pages [split $pages -]
					if {![regexp {^([0-9]+)\(([0-9]+)\)$} $vol temp vol subvol]} {
						lappend ref class journal source $jou volume $vol \
						pages_start [lindex $pages 0] pages_end [lindex $pages 1] year $year
					} else {
						lappend ref class journal source $jou volume $vol subvolume $subvol \
						pages_start [lindex $pages 0] pages_end [lindex $pages 1] year $year
					}
				} elseif [regexp {^Thesis \(([0-9?]+)\), (.*)$} $value temp year place] {
					regsub -all " *\n *" $place { } place
					lappend ref class thesis place $place year $year
				} elseif [regexp {^Patent number ([^.-]+)-([^/]+)/([^,]+), ([^.]+)\..(.*)} $value temp patent type serial date applicants] {
					set date [time_format [time_scan $date] "%Y-%m-%d"]
					set applicants [string_change [string trimright $applicants " ."] [list "., " ".#" ",\n" ".#" ". and " ".#" "\;" ""]]
					set applicants [split $applicants "#"]
					lappend ref class patent patent $patent type $type serial $serial date $date applicants $applicants
				} else {
					lappend ref class unknown source $value
				}
				lappend data(reflist) $refnum [get refpos {}] $ref
				set ref {}
			}
			COMMENT {
				regsub -all " *\n *" $value { } value
				set data(comment) $value
			}
			FEATURES {
				set data(ftstart) [file_linestart $f $line]
				# read a line and the continuation lines
				while {![eof $f]} {
					set line [gets $f]
					if {![string_equal [string index $line 0] " "]} break
				}
				set data(_ftend) [expr {[tell $f] - [string length $line] - 1}]
				set data(_ftlen) [expr {$data(_ftend) - $data(ftstart)}]
			}
			"" {
				break
			}
			default {
				log "unknown key $lkey ($value)"
				set data([get trans($lkey) $lkey]) $value
			}
		}
	}
	if {![info exists data(acc)]} {
		error eof
	}
	# Go for the sequence
	set data(seqstart) [tell $f]
	if {[string_equal $lkey ORIGIN]} {
		# Go for the sequence
		set seq ""
		while {![eof $f]} {
			set line [gets $f]
			if {"$line" == "//"} break
			append seq $line
		}
		set data(seqend) [tell $f]
		regsub -all {[^A-Za-z]} $seq {} seq
		set len [string length $seq]
	} else {
		set data(seqend) 0
		set line [gets $f]
		set len -1
		set seq $value
	}
	set data(length) $len
}

GenbankFile method open {file} {
	private $object f num idarray
	if {[info exists f]} {close $f}
	unset -nocomplain data
	unset -nocomplain idarray
	set f [open $file]
	set num 0
	set result {}
	while {![eof $f]} {
		if {[catch {$object _parseentry $num} error]} {
			if {[string equal $error eof]} break
			error $error
		}
		set acc [$object get $num acc]
		lappend result $acc
		set idarray($acc) $num
		log "$num $acc"
		incr num
	}
	return $result
}

GenbankFile method get {id field} {
	if {![isint $id]} {
		private $object idarray
		set id $idarray($id)
	}
	upvar #0 [privatevar $object data_$id] data
	return $data($field)	
}

GenbankFile method names {id} {
	if {![isint $id]} {
		private $object idarray
		set id $idarray($id)
	}
	upvar #0 [privatevar $object data_$id] data
	return [array names data]
}

GenbankFile method list {{field acc}} {
	private $object f num
	set result {}
	for {set i 0} {$i < $num} {incr i} {
		upvar #0 [privatevar $object data_$i] data
		lappend result [get data($field) ""]
	}
	return $result
}

GenbankFile method fields {} {
	private $object f num
	set result {}
	for {set i 0} {$i < $num} {incr i} {
		upvar #0 [privatevar $object data_$i] data
		eval list_addnew result [array names data]
	}
	return $result
}

GenbankFile method fsequence {id} {
	private $object f
	if {![isint $id]} {
		private $object idarray
		set id $idarray($id)
	}
	upvar #0 [privatevar $object data_$id] data
	seek $f $data(seqstart)
	return [file_getrange $f $data(seqstart) $data(seqend)]
}

GenbankFile method sequence {id} {
	set seq [$object fsequence $id]
	regsub -all {[^A-Za-z]} $seq {} seq
	return $seq
}

GenbankFile method ffeatures {id} {
	private $object f
	if {![isint $id]} {
		private $object idarray
		set id $idarray($id)
	}
	upvar #0 [privatevar $object data_$id] data
	seek $f $data(ftstart)
	return [read $f $data(_ftlen)]
}

GenbankFile method _parselocation {location args} {
	set result ""
	set operator [string index $location 0]
	if {[inlist {c j o} $operator]} {
		set len [string length $location]
		incr len -1
		set operator [string index $location 0]
		set pos [string first "\(" $location]
		set location [string range $location [expr {$pos+1}] [expr {$len-1}]]
		switch -- $operator {
			{c} {
				set result ""
				set parts [$object _parselocation $location]
				set i [llength $parts]
				for {incr i -1} {$i >= 0} {incr i -1} {
					set part [lindex $parts $i]
					if {[lsearch $part complement] != -1} {
						set part [map_unset $part complement]
					} else {
						lappend part complement 1
					}
					lappend result $part
				}
			}
			{j} - {o} {
				if [llength $args] {return ""}
				foreach l [split $location ", "] {
					if ![llength $l] continue
					eval lappend result [$object _parselocation $l]
				}
			}
		}
		return $result
	} else {
		set result ""
		if [regexp {^([^:]+):(.*)$} $location temp acc loc] {
			regsub "\\..*$" $acc {} acc
			lappend result acc $acc
			set location $loc
		}
		set loc [string_change $location {: { : } ( {} ) {} .. { - } {. .} { - } ^ { ^ } . { . }}]
		set current start
		foreach {value sep} $loc {
			if {[inlist {< >} [string index $value 0]]} {
				set partial 1
				set value [string range $value 1 end]
			} else {
				set partial 0
			}
			switch -exact -- $sep {
				- {
					set ${current}_partial $partial
					set $current $value
					set current end
				}
				^ {
					set start $value
					set length 0
					set current end
				}
				. {
					set ${current}_partial $partial
					set $current $value
					set current ${current}_span
				}
				{} {
					set ${current}_partial $partial
					set $current $value
				}
				default {
					error "Couldn't correctly parse location \"$location\""
				}
			}
		}
		if {![info exists end]} {set end $start}
		if {[info exists length]} {
			lappend result start $start end $start
			set span [expr {$end-$start}]
			if {$span} {
				lappend result start_span $span
			}
		} else {
			incr start -1
			incr end -1
			if {![info exists end]} {
				lappend result start $start end [expr {$start+1}]
			} else {
				lappend result start $start end $end
			}
			if {[info exists start_span]} {
				lappend result start_span [expr {$start_span-$start}]
			}
			if {[info exists end_span]} {
				lappend result end_span [expr {$end_span-$end+1}]
			}
		}
		if {[get start_partial 0] || [get start_span_partial 0]} {
			if {[get end_partial 0] || [get end_span_partial 0]} {
				lappend result partial b
			} else {
				lappend result partial s
			}
		} elseif {[get end_partial 0] || [get end_span_partial 0]} {
			lappend result partial e
		}
		return [list $result]
	}
}

GenbankFile method _parsefeatures {} {
	private $object f
	set ftlist {}
	set ftel {}
	set line [gets $f]
	while {![eof $f]} {
		set cline $line
		# read a line and the continuation lines
		while {![eof $f]} {
			set line [gets $f]
			if {![string_equal [string index $line 0] { }]} break
			if {![string_equal [string index $line 5] { }]} break
			if {[string_equal [string index $line 21] "/"]} break
			append cline " [string range $line 21 end]"
		}
		# check for the end of the current feature
		if {![string_equal [string index $cline 5] { }]} {
			# handle location of feature
			set key [string trimright [string range $cline 5 19]]
			set location [string range $cline 21 end]
			set loc [$object _parselocation $location]
			if {[llength $ftel]} {
				lappend ftel [array get attrs]
				lappend ftlist $ftel
				set ftel {}
			}
			unset -nocomplain attrs
			lappend ftel $key $loc
		} else {
			# handle attributes to the current feature
			set pos [string first = $cline]
			if {$pos != -1} {
				set attr [string range $cline 22 [expr {$pos-1}]]
				if ![string length $attr] continue
				set value [string trimleft [string trimright [string range $cline [expr {$pos+1}] end] "\""] "\""]
				switch -- $attr {
					translation {
						regsub -all " " $value "" value
						set attrs(proteinseq) $value
					}
					citation {
						regexp {\[([0-9]+)\]} $value temp value
						if [info exists rn($value)] {
							set attrs(refnum) $value
						} else {
							set attrs($attr) $value
						}
					}
					organism {
						set attrs(organism) $value
					}
					default {
						lappend attrs($attr) $value
					}
				}
			} else {
				set attr [string range $cline 22 end]
				lappend attrs(switches) $attr
			}
		}
		# check for the end of the feature list
		if {![string_equal [string index $line 0] " "]} break
	}
	if {[llength $ftel]} {
		lappend ftel [array get attrs]
		lappend ftlist $ftel
	}
	return $ftlist
}

GenbankFile method features {id} {
	private $object f
	if {![isint $id]} {
		private $object idarray
		set id $idarray($id)
	}
	upvar #0 [privatevar $object data_$id] data
	seek $f $data(ftstart)
	return [$object _parsefeatures]
}

