proc geneid_run {source dest args} {
	set opt(-param) human3iso
	cmd_args bio_geneid {
		-param {any "parameters to use (human1iso, human3iso, dros) or name of parameter file"}
	} {} $args
	set geneid $::bio::external(geneid)
	set geneiddir $::bio::external(geneiddir)
	if {[inlist {human1iso human3iso dros} $opt(-param)]} {
		set paramfile [file join $geneiddir param $opt(-param).param]
	} else {
		if {![file exists $opt(-param)]} {error "file \"$opt(-param)\" does not exists"}
		set paramfile $opt(-param)
	}
	set f [open $dest.runinfo w]
	puts $f [list source $source]
	puts $f [list time [clock format [clock seconds] -format "%Y-%m-%d %H:%M:%S"]]
	puts $f [list paramfile $opt(-param)]
	if [catch {exec $geneid -XG -P $paramfile $source > $dest} result] {
		error "error from geneid: $result"
	}
	close $f
}

proc biodb_import_geneid {biodb source} {
	if {[file exists $source.runinfo]} {
		set c [file_read $source.runinfo]
	} else {
		set c {}
	}
	$biodb commit
	$biodb begin
	if [catch {
		set eventid [$biodb event type import sourcetpye geneid source $source options $c]
		set num 0
		set fts {}
		set found -1
		set f [open $source]
		while 1 {
			set line [gets $f]
			if {([string equal [string index $line 0] #]) || [eof $f]} {
				set len [llength $fts]
				if {$len > 1} {
					set ft [$biodb new geneid]
					if {[string_equal $strand -]} {set fts [list_reverse $fts]}
					$biodb set $ft ftkey CDS start $start end $end complement $complement seq [lindex $seqid 1] \
						geneid_score $genescore
					$biodb ftparts $ft $fts
				}
				set fts {}
				incr found
				if {[eof $f]} break
				if {![string equal [string range $line 1 5] { Gene}]} continue
				if {![regexp {Gene ([0-9]+) *\((Forward|Reverse)\).* Score = ([0-9.]+)} $line temp genenum strand genescore]} {
					error "error in format for line: \"$line\""
				}
			} else {
				set type [lindex $line 2]
				switch $type {
					Single - First - Internal - Terminal {
						set ft [$biodb new geneid]
						foreach {seqname gffsource feature start end score strand frame gene} $line break
						if [catch {$biodb scanid $seqname} seqid] {
							set seqid [$biodb scanid $source]
						}
						if {[string equal $feature Single]} {
							set ftkey CDS
						} else {
							set ftkey exon
						}
						if {[string_equal $strand -]} {
							set complement 1
						} else {
							set complement 0
						}
						incr start -1
						incr end -1
						log $ft:$start-$end
						$biodb set $ft event [lindex $eventid 1] ftkey $ftkey \
							seq [lindex $seqid 1] start $start end $end complement $complement \
							geneid_score $score frame $frame \
							feature [string index $feature 0]
						lappend fts $ft
					}
				}
			}
		}
		close $f
	} result] {
		$biodb rollback
		return -code error -errorinfo $::errorInfo $result
	} else {
		$biodb commit
	}
	return $eventid
}

proc bio_geneid {args} {
	set opt(-param) human3iso.param
	set opt(-cacheresult) {}
	cmd_args bio_geneid {
		-param {any "parameters to use (human1iso, human3iso, dros) or name of parameter file"}
		-cacheresult {filename "save resultfile here"}
	} "db source" $args
	# $db class add geneid
	if {[string length $opt(-cacheresult)]&&[file exists $opt(-cacheresult)]} {
		set tempdst $opt(-cacheresult)
	} else {
		set geneid $::bio::external(geneid)
		set geneiddir $::bio::external(geneiddir)
		if {[inlist {human1iso human3iso dros} $opt(-param)]} {
			set paramfile [file join $geneiddir param $opt(-param).param]
		} else {
			if {![file exists $opt(-param)]} {error "file \"$opt(-param)\" does not exists"}
			set paramfile $opt(-param)
		}
		set tempsrc [tempfile]
		set tempdst [tempfile]
		set f [open $tempsrc w]
		puts $f ">work"
		puts $f [$db get $source dnaseq]
		close $f
		if [catch {exec $geneid -XG -P $paramfile $tempsrc > $tempdst} result] {
			error "error from geneid: $result"
		}
		file delete $tempsrc
	}
	set eventid [biodb_import_geneid $db $tempdst]
	$db set eventid type run source $source
	if {[string length $opt(-cacheresult)]} {
		if {![file exists $opt(-cacheresult)]} {
			file rename $tempdst $opt(-cacheresult)
		}
	} else {
		file delete $tempdst
	}
	return $eventid
}
