proc bio_matrix {name} {
	upvar #0 ::bio::matrix matrix
	if {![info exists matrix($name)]} {
		if {[file exists $name]} {
			set file $name
		} elseif {[file exists $bio::dir/conf/matrices/$name.mat]} {
			set file $bio::dir/conf/matrices/$name.mat
		} else {
			error "matrix $name not found in $bio::dir/conf/matrices"
		}
		set data [split [file_read $file] \n]
		set type [lindex $data 0]
		if {[inlist {Distances Similarity} $type]} {
			set pos 0
			set len [llength $data]
			while {$pos < $len} {
				incr pos
				set line [lindex $data $pos]
				set list [split $line :]
				if {[llength $list] != 2} break
				set def([lindex $list 0]) [lindex $list 1]
			}
			set data [lrange $data $pos end]
			if {$type eq "Distances"} {
				set gap [get opt(-gap) $def(gap)]
				set indel [get opt(-indel) $def(indel)]
				set profile_indel [get opt(-profile_indel) ${def(profile indel)}]
			} else {
				set gap [expr {-[get opt(-gap) $def(gap)]}]
				set indel [expr {-[get opt(-indel) $def(indel)]}]
				set profile_indel [expr {-[get opt(-profile_indel) ${def(profile indel)}]}]
			}
			set anymismatch -4
			set anymatch 5
		} else {
			set type Similarity
			if {[inlist {b p} [string index $name 0]]} {
				set gap [expr {-[get opt(-gap) 10]}]
				set indel [expr {-[get opt(-indel) 2]}]
				set profile_indel [expr {-[get opt(-profile_indel) 3]}]
			} else {
				set gap [expr {-[get opt(-gap) 14]}]
				set indel [expr {-[get opt(-indel) 4]}]
				set profile_indel [expr {-[get opt(-profile_indel) 9]}]
			}
			set poss [list_find -glob $data {#*}]
			set data [list_sub $data -exclude $poss]
			set data [list_remove $data {}]
			if {[lindex $data 0 end] eq "*"} {
				set anymismatch [lindex $data 1 end]
				set anymatch [lindex $data end end]
			} else {
				set anymismatch -4
				set anymatch 5
			}
		}
		set temp [list $type \
				[list gap $gap] \
				[list indel $indel] \
				[list profile_indel $profile_indel] \
				[list anymismatch $anymismatch] \
				[list anymatch $anymatch] \
				{}]
		eval lappend temp $data
		set matrix($name) $temp
	}
	return $matrix($name)
}
