proc newcpg {db source} {
	# $db class add newcpg
	set embossbin $::bio::external(embossbin)
	set emboss $::bio::external(emboss)
	set tempsrc [tempfile]
	set tempdst [tempfile]
	set f [open $tempsrc w]
	puts $f ">work;"
	puts $f [$db get $source dnaseq]
	close $f
	catch {exec $embossbin/newcpgreport -sequence $tempsrc -outfile $tempdst \
		-window 100 -shift 1 -minlen 200 -minoe 0.6 -minpc 50} result
	if {![string_equal $result {Report CpG rich areas}]} {
		error "error from newcpgreport: $result"
	}
	set eventid [biodb_import_newcpg $db $tempdst]
	$db set $eventid type run scouretype newcpg options [list window 100 shift 1 minlen 200 minoe 0.6 minpc 50] source $source
	file delete $tempsrc
	file delete $tempdst
	return $eventid
}

proc newcpg_run {source dest} {
	set embossbin $::bio::external(embossbin)
	set emboss $::bio::external(emboss)
	set f [open [file join $dest.runinfo] w]
	puts $f {window 100}
	puts $f {shift 1}
	puts $f {min_len 200}
	puts $f {min_obs_exp 0.6}
	puts $f {min_gc 50}
	puts $f [list source $source]
	puts $f [list time [clock format [clock seconds] -format "%Y-%m-%d %H:%M:%S"]]
	close $f
	catch {exec $embossbin/newcpgreport -sequence $source -outfile $dest \
		-window 100 -shift 1 -minlen 200 -minoe 0.6 -minpc 50} result
	if {![string_equal $result {Report CpG rich areas}]} {
		error "error from newcpgreport: $result"
	} 
}

proc biodb_import_newcpg {biodb source} {
	if {[file exists $source.runinfo]} {
		set c [file_read $source.runinfo]
	} else {
		set c {}
	}
	$biodb begin
	if [catch {
		set eventid [$biodb event type import sourcetpye newcpg source $source options $c]
		set list [list_remove [string_split [file_read $source] //\n] {}]
		foreach c $list {
			if {[string equal $c {}]} continue
			regexp {^ID   ([^\n]+)} $c temp temp
			if [catch {$biodb scanid $temp acc} id] {
				set id [$biodb scanid $source acc]
			}
			log "$id.$acc"
			set pos [string first "FT   " $c]
			set c [string range $c $pos end]
			regsub -all {[^0-9.]} $c { } datalist
			set num 0
			foreach {loc size sum_gc percent_gc obsexp} $datalist {
				if {[string_equal $size {}]} break
				set ft [$biodb new newcpg]
				foreach {start end} [string_split $loc ..] break
				incr start -1
				incr end -1
				$biodb set $ft event [lindex $eventid 1] \
					seq $id ftkey cpg_island start $start end $end complement 0 \
					sum_gc $sum_gc percent_gc $percent_gc obsexp $obsexp
				incr num
				log "$ft $id cpg $start-$end"
			}
		}
	} result] {
		$biodb rollback
		return -code error -errorinfo $::errorInfo $result
	} else {
		$biodb commit
	}
	return $eventid
}

