package require Extral
package require http
package require tdom
if { ![package vsatisfies [package provide Tcl] 8.5] } {package require dict}

#
proc pubmed_query {query {retmax 1000}} {
	set q [::http::formatQuery db pubmed term $query retmax $retmax tool testcode]
	set token [http::geturl eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?$q]
	set data [::http::data $token]
	::http::cleanup $token
	dom parse $data dom
	$dom documentElement doc
	set result {}
	foreach node [$doc selectNodes /eSearchResult/IdList/Id/text()] {
		lappend result [$node nodeValue]
	}
	return $result
}

proc pubmed_getdata {pubmed} {
	upvar #0 ::bio::pubmeda pubmeda
	if {[info exists pubmeda($pubmed)]} {
		return $pubmeda($pubmed)
	}
	set q [::http::formatQuery db pubmed id $pubmed retmode xml rettype citation tool testcode]
	set token [http::geturl eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?$q]
	set data [::http::data $token]
	::http::cleanup $token
	set pubmeda($pubmed) $data
	return $data
}

proc pubmed_xml2dict {data} {
	dom parse $data dom
	$dom documentElement doc
	set base /PubmedArticleSet/PubmedArticle/MedlineCitation
	set authors {}
	set authorlist {}
	foreach node [$doc selectNodes [subst {$base/Article/AuthorList//Author}]] {
		set lastname [$node selectNodes {string(LastName)}]
		set firstname [$node selectNodes {string(ForeName)}]
		set initials [$node selectNodes {string(Initials)}]
		lappend authorlist $lastname $firstname $initials
		lappend authors "$lastname,[join [split $initials {}] .]."
	}
	set authors [join $authors ", "]
	set result [dict create authors $authors authorlist $authorlist]
	foreach {key path} {
		title Article/ArticleTitle
		year Article/Journal/JournalIssue/PubDate/Year
		month Article/Journal/JournalIssue/PubDate/Month
		day Article/Journal/JournalIssue/PubDate/Day
		volume Article/Journal/JournalIssue/Volume
		issue Article/Journal/JournalIssue/Issue
		pages Article/Pagination/MedlinePgn
		journal MedlineJournalInfo/MedlineTA
		abstract Article/Abstract/AbstractText
		affiliation Article/Affiliation
	} {
		dict set result $key [$doc selectNodes [subst {string($base/$path)}]]
	}
	return $result
}
