proc repeatmasker_run {source dest} {
	set repeatmasker $::bio::external(repeatmasker)
	file mkdir $dest
	set f [open [file join $dest.runinfo] w]
	puts $f [list source $source]
#	puts $f [list options { -gff -poly -u -xm}]
	puts $f [list options {}]
	puts $f [list time [clock format [clock seconds] -format "%Y-%m-%d %H:%M:%S"]]
	close $f
	file copy $source $dest/$source
	exec $repeatmasker $dest/$source
	file delete $dest/$source
}

proc biodb_import_repeatmasker {biodb source} {
	if {[file exists $source.runinfo]} {
		set c [file_read $source.runinfo]
	} else {
		set c {}
	}
	if {[string equal [file extension $source] .out]} {
		set files $source
	} else {
		set files [glob $source/*.out]
	}
	$biodb begin
	if [catch {
		set eventid [$biodb event type import sourcetype repeatmasker source $source options $c]
		set num 0
		foreach file $files {
			set f [open $file]
			gets $f
			gets $f
			gets $f
			while {![eof $f]} {
				set line [gets $f]
				if {[string equal $line {}]} continue
				foreach {score divergence deletions insertions id start end temp strand repeat rpt_family repeat_start repeat_end temp repeat_id} $line break
				if [catch {$biodb scanid $id acc} id] {
					set id [$biodb scanid [file root $source] acc]
				}
				incr start -1
				incr end -1
				regexp {[0-9]+} $repeat_start repeat_start
				regexp {[0-9]+} $repeat_end repeat_end
				incr repeat_start -1
				incr repeat_end -1
				if {[string_equal $strand C]} {set complement 1} else {set complement 0}
				set ft [$biodb new repeatmasker]
				$biodb set $ft event [lindex $eventid 1] \
					seq [lindex $id 1] ftkey repeat_region start $start end $end complement $complement \
					type $repeat rpt_family $rpt_family \
					divergence $divergence deletions $deletions insertions $insertions \
					repeatmasker_score $score repeat_id $repeat_id	\
					repeat_start $repeat_start repeat_end $repeat_end
				log "$num.$id $repeat"
				incr num
			}
			close $f
		}
	} result] {
		$biodb rollback
		return -code error -errorinfo $::errorInfo $result
	} else {
		$biodb commit
	}
}

proc bio_repeatmasker {db source} {
	# $db class add repeatmasker
	set repeatmasker $::bio::external(repeatmasker)
	set tempsrc [tempfile]
	set f [open $tempsrc w]
	puts $f ">work;"
	puts $f [$db get $source dnaseq]
	close $f
	exec $repeatmasker $tempsrc
	set eventid [$db event type run sourcetype repeatmasker source $source]
		date [clock format [clock seconds] -format "%Y-%m-%d %H:%M:%S"]

	set f [open $tempsrc.out]
	gets $f
	gets $f
	gets $f
	while {![eof $f]} {
		set line [gets $f]
		foreach {score divergence deletions insertions id start end temp strand repeat rpt_family} $line break
		set ft [$db new ft]
		set source [lindex [split $id |] end]
		$db set $ft ftkey repeat_region _source $source analysis repeatmasker event [lindex $eventid 1]
		set length [expr {$end-$start+1}]
		incr start -1
		incr end -1
		if {[string_equal $strand C]} {
			$db set $ft complement 1
		}
		foreach key {start end length repeat rpt_family divergence deletions insertions} {
			$db set $ft $key [set $key]
		} 
	}
	file delete $tempsrc
	return [llength $c]
}

proc repeatmasker_parse {file} {
	set fresult {}
	set f [open $file]
	gets $f
	gets $f
	gets $f
	while {![eof $f]} {
		set result {}
		set line [gets $f]
		foreach {score divergence deletions insertions id start end temp strand repeat rpt_family} $line break
		lappend result ftkey repeat_region id $id analysis repeatmasker
		set length [expr {$end-$start+1}]
		incr start -1
		incr end -1
		if {[string_equal $strand C]} {
			lappend result complement 1
		}
		foreach key {start end length repeat rpt_family divergence deletions insertions} {
			lappend result $key [set $key]
		}
		lappend fresult $result
	}
	return $fresult
}

#set file data/out.fasta.out
#set data [parse_repeatmasker $file]

proc conv_repeatmasker_biodb {args} {
	cmd_args conv_repeatmasker_biodb {
		-runinfo {filename "file with runinfo"}
		-seq {integer "seq id"}
	} {db file} $args
	set eventid [$object event type import sourcetype repeatmasker source $file]
	set f [open $file]
	gets $f
	gets $f
	gets $f
	set found 0
	while {![eof $f]} {
		set result {}
		set line [gets $f]
		foreach {
			score divergence deletions insertions query start end temp strand
			repeat rpt_family repeat_start repeat_end temp repeat_id
		} $line break
		lappend result ftkey repeat_region id $id analysis repeatmasker
		set length [expr {$end-$start+1}]
		incr start -1
		incr end -1
		if {[string_equal $strand C]} {
			lappend result complement 1
		}
		set rid [$db new ft]
		$db set $rid start $start end $end \
			repeat $repeat \
			rpt_family $rpt_family \
			divergence $divergence \
			deletions $deletions \
			insertions $insertions \
			repeat $repeat_start \
			repeat_end temp repeat_id
		foreach key {start end length repeat rpt_family divergence deletions insertions} {
			lappend result $key [set $key]
		}
		lappend fresult $result
	}
}
