proc bio_getaccns {args} {
	set result [list]
	foreach file $args {
		set gzip [string_equal [file extension $file] .gz]
		if $gzip {
			exec gunzip $file
			set file [file root $file]
		}
		set f [open $file]
		while {![eof $f]} {
			set line [gets $f]
			switch -glob -- $line {
				{AC   *} {
					regexp {AC   ([A-Z0-9]+)} $line temp accn
					lappend result $accn
				}
				{acc:*} {
					regexp {acc:([A-Z0-9]+)} $line temp accn
					lappend result $accn
				}
				{ACCESSION   *} {
					regexp {ACCESSION   ([A-Z0-9]+)} $line temp accn
					lappend result $accn
				}
			}
		}
		close $f
		if $gzip {exec gzip $file}
	}
	return [list_remdup $result]
}

proc neof {f} {
	upvar #0 bio::nread($f) buffer
	set result {}
	if {[info exists buffer]} {
		return 0
	} else {
		return [eof $f]
	}
}

proc nread {f {size 0}} {
	upvar #0 bio::nread($f) buffer
	if {[info exists buffer]} {
		if {$size == 0} {
			set pos [string first \n $buffer]
			if {$pos == -1} {
				append buffer [gets $f]
			}
		} else {
			append buffer [read $f [expr {$size-[string length $buffer]}]]
		}
	} else {
		if {$size == 0} {
			set buffer [gets $f]
		} else {
			set buffer [read $f $size]
		}
	}
	set pos [string first \n $buffer]
	if {$pos != -1} {
		set result [string range $buffer 0 [expr {$pos-1}]]
		set buffer [string range $buffer [expr {$pos+1}] end]
	} else {
		set result $buffer
		unset buffer
	}
	return $result
}

proc channel {data} {
	set channel [memchan]
	puts -nonewline $channel $data
	fconfigure $channel -eofchar \0
	puts -nonewline $channel \0
	seek $channel 0
	return $channel
}

proc file_linestart {f line} {
	set keep [tell $f]
	set keepeof [eof $f]
	set pos [expr {$keep - 1}]
	while 1 {
		seek $f $pos
		set c [read $f 1]
		if {![inlist [list \c \n] $c]} break
		incr pos -1
	}
	set pos [expr {$pos-[string length $line]+1}]
	seek $f $keep
	if {$keepeof} {
		read $f 1
	}
	return $pos
}

proc file_getrange {f start end} {
	set keep [tell $f]
	set keepeof [eof $f]
	set confkeep [fconfigure $f]
	fconfigure $f -encoding binary -translation binary
	seek $f $start
	set result [read $f [expr {$end-$start}]]
	eval {fconfigure $f} $confkeep
	seek $f $keep
	if {$keepeof} {
		read $f 1
	}
	return $result
}

proc prali {refseq seq r} {
	foreach {seqnum refstart refend start end len rali} $r break
	set seq1 [string range [lindex $refseq $seqnum] $refstart $refend]
	set seq2 [string range $seq $start $end]
	# start inserting at the end (reverse rali)
	foreach {len pos reflen refpos} [list_reverse $rali] {
		if {$reflen} {
			set pos [expr {$pos-$start}]
			set seq2 [string_replace $seq2 $pos -1 [string_fill - $reflen]]
		}
		if {$len} {
			set refpos [expr {$refpos-$refstart}]
			set seq1 [string_replace $seq1 $refpos -1 [string_fill - $len]]
		}
	}
	pali [list $seq1 $seq2]
}

proc pali {ali {end end}} {
	foreach {a1 a2} $ali break
	set all {}
	foreach a $a1 {
		regsub -all [string_fill . 70] [string range $a 0 $end] {\0 } a
		lappend all $a
	}
	foreach a $a2 {
		regsub -all [string_fill . 70] [string range $a 0 $end] {\0 } a
		lappend all $a
	}
	set i 0
	while 1 {
		if {[lindex $all 0 $i] eq ""} break
		foreach list $all {
			puts [lindex $list $i]
		}
		puts ""
		incr i
	}
}

proc max args {
	set result [lindex $args 0]
	foreach el [lrange $args 1 end] {
		if {$el > $result} {set result $el}
	}
	return $result
}
