# Initialisation of the BioTcl package
#
# Copyright (c) 1996 Peter De Rijk
#
# See the file "README.txt" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# =============================================================

namespace eval ::bio {}

package require Extral
package require Class
if { ![package vsatisfies [package provide Tcl] 8.5] } {package require dict}
#package require Memchan

# $Format: "set ::bio::version $ProjectMajorVersion$.$ProjectMinorVersion$"$
set ::bio::version 0.1
# $Format: "set ::bio::patchlevel $ProjectPatchLevel$"$
set ::bio::patchlevel 2

package provide BioTcl $::bio::version

package require pkgtools
pkgtools::init $bio::dir biotcl ali_align

proc bio::findexternal {prog template args} {
	global tcl_platform
	upvar ::bio::external external
	set externdir [file join $::bio::dir extern]
	set externdirlocal [file join [file dir [file dir $::bio::dir]] extern]
	set list {}
	lappend list [file join $::bio::dir extern]/$template
	lappend list [file join [file dir [file dir $::bio::dir]] extern]/$template
	lappend list /usr/local/bin/$template
	lappend list ~/bin/$template
	eval lappend list $args
	foreach file $list {
		if {[file exists $file]} {
			set external($prog) $file
			return $file
		}
		if {($tcl_platform(platform) eq "windows") && [file exists $file.exe]} {
			set external($prog) $file
			return $file
		}
	}
	if {![catch {exec which $prog} file]} {
		set external($prog) $file
		return $file
	}
	set external($prog) [lindex $args 0]
	return $external($prog)
}

bio::findexternal genscan genscan/genscan
set ::bio::external(genscandir) [file dir $bio::external(genscan)]
bio::findexternal geneid geneid/bin/geneid
set ::bio::external(geneiddir) [file dir [file dir $bio::external(geneid)]]
bio::findexternal emboss EMBOSS /usr/local/share/EMBOSS
bio::findexternal embossbin EMBOSS/bin /usr/local/bin
bio::findexternal sputnik sputnik/sputnik
bio::findexternal repeatmasker repeatmasker/RepeatMasker
bio::findexternal blast blast
bio::findexternal netblast netblast/blastcl3
bio::findexternal wisedir wise2.1.23c/bin/
bio::findexternal yass yass-1.08/yass

#array set ::bio::external [file_read [file join $::bio::dir conf external.lst]]

proc log log {puts stdout $log}
proc setlog {def} {
	proc log log $def
}
proc setlogchannel {channel} {
	if {[string length $channel]} {
		setlog [subst {puts $channel \$log}]
	} else {
		setlog {}
	}
	proc log log $def
}

#
# The lib dir contains the Tcl code defining the public biotcl
# functions. The lib dir is added to the auto_path so that
# these functions will be loaded on demand.
#

lappend auto_path [file join ${::bio::dir} lib code] [file join ${::bio::dir} lib interface] [file join ${::bio::dir} lib biodb]

